@@ -50,13 +50,13 @@ from robotoff.off import (
     get_product,
     get_server_type,
 )
-from robotoff.prediction.category.neural.category_classifier import CategoryClassifier
+from robotoff.prediction.category import predict_category
 from robotoff.prediction.object_detection import ObjectDetectionModelRegistry
 from robotoff.prediction.ocr.dataclass import OCRParsingException
 from robotoff.prediction.types import PredictionType
 from robotoff.products import get_product_dataset_etag
 from robotoff.spellcheck import SPELLCHECKERS, Spellchecker
-from robotoff.taxonomy import TaxonomyType, get_taxonomy, match_taxonomized_value
+from robotoff.taxonomy import is_prefixed_value, match_taxonomized_value
 from robotoff.utils import get_image_from_url, get_logger, http_session
 from robotoff.utils.es import get_es_client
 from robotoff.utils.i18n import TranslationStore
@@ -89,7 +89,7 @@ def _get_skip_voted_on(
 
 
 ###########
-# IMPORTANT: remember to update documentation at doc/references/api.md if you change API
+# IMPORTANT: remember to update documentation at doc/references/api.yml if you change API
 ###########
 
 
@@ -243,7 +243,7 @@ def device_id_from_request(req: falcon.Request) -> str:
 
 class AnnotateInsightResource:
     def on_post(self, req: falcon.Request, resp: falcon.Response):
-        insight_id = req.get_param("insight_id", required=True)
+        insight_id = req.get_param_as_uuid("insight_id", required=True)
         annotation = req.get_param_as_int(
             "annotation", required=True, min_value=-1, max_value=1
         )
@@ -403,28 +403,34 @@ class OCRInsightsPredictorResource:
 class CategoryPredictorResource:
     @jsonschema.validate(schema.PREDICT_CATEGORY_SCHEMA)
     def on_post(self, req: falcon.Request, resp: falcon.Response):
-        """Predict categories using neural categorizer for a specific product."""
+        """Predict categories using neural categorizer and matching algorithm
+        for a specific product."""
+        predictors: List[str] = req.media.get("predictors") or ["neural", "matcher"]
+
         if "barcode" in req.media:
             # Fetch product from DB
             barcode: str = req.media["barcode"]
-            product = get_product(barcode, fields=["product_name", "ingredients_tags"])
+            product = get_product(barcode) or {}
+            if not product:
+                raise falcon.HTTPNotFound(description=f"product {barcode} not found")
         else:
             product = req.media["product"]
-        deepest_only: bool = req.media.get("deepest_only", False)
-        threshold: Optional[float] = req.media.get("threshold")
+            if "matcher" in predictors:
+                if "lang" not in req.media:
+                    raise falcon.HTTPBadRequest(
+                        description="lang field is required when using matcher predictor"
+                    )
+                lang = req.media["lang"]
+                product[f"product_name_{lang}"] = product["product_name"]
+                product["languages_codes"] = [lang]
 
-        categories = []
-
-        if product:
-            predictions = CategoryClassifier(
-                get_taxonomy(TaxonomyType.category.name)
-            ).predict(product, deepest_only, threshold)
-            categories = [
-                {"value_tag": p.value_tag, "confidence": p.data["confidence"]}
-                for p in predictions
-            ]
-
-        resp.media = {"neural": categories}
+        resp.media = predict_category(
+            product,
+            neural_predictor="neural" in predictors,
+            matcher_predictor="matcher" in predictors,
+            deepest_only=req.media.get("deepest_only", False),
+            threshold=req.media.get("threshold"),
+        )
 
 
 class UpdateDatasetResource:
@@ -647,16 +653,32 @@ class ImageLogoSearchResource:
         type_: Optional[str] = req.get_param("type")
         barcode: Optional[str] = req.get_param("barcode")
         value: Optional[str] = req.get_param("value")
+        taxonomy_value: Optional[str] = req.get_param("taxonomy_value")
         min_confidence: Optional[float] = req.get_param_as_float("min_confidence")
         random: bool = req.get_param_as_bool("random", default=False)
         server_domain: Optional[str] = req.get_param("server_domain")
         annotated: Optional[bool] = req.get_param_as_bool("annotated")
 
+        if type_ is None and (value is not None or taxonomy_value is not None):
+            raise falcon.HTTPBadRequest(
+                description="a type is required if `value` or `taxonomy_value` is provided"
+            )
+
+        if value is not None and taxonomy_value is not None:
+            raise falcon.HTTPBadRequest(
+                description="`value` and `taxonomy_value` are mutually exclusive parameters"
+            )
+
+        if type_ == "label" and taxonomy_value is None and value is not None:
+            raise falcon.HTTPBadRequest(
+                description="you should provide a `taxonomy_value` and not a `value` "
+                "for label type"
+            )
+
         where_clauses = []
         if annotated is not None:
             where_clauses.append(LogoAnnotation.annotation_value.is_null(not annotated))
 
-        where_clauses = [LogoAnnotation.annotation_value.is_null(not annotated)]
         join_image_prediction = False
         join_image_model = False
 
@@ -681,6 +703,9 @@ class ImageLogoSearchResource:
             value_tag = get_tag(value)
             where_clauses.append(LogoAnnotation.annotation_value_tag == value_tag)
 
+        if taxonomy_value is not None:
+            where_clauses.append(LogoAnnotation.taxonomy_value == taxonomy_value)
+
         query = LogoAnnotation.select()
         join_image_prediction = join_image_prediction or join_image_model
 
@@ -780,6 +805,10 @@ class ImageLogoAnnotateResource:
                 raise falcon.HTTPNotFound(description=f"logo {logo_id} not found")
 
             if value is not None:
+                if type_ == "label" and not is_prefixed_value(value):
+                    raise falcon.HTTPBadRequest(
+                        description=f"language-prefixed value are required for label type (here: {value})"
+                    )
                 logo.annotation_value = value
                 value_tag = get_tag(value)
                 logo.annotation_value_tag = value_tag
@@ -911,6 +940,7 @@ class ProductQuestionsResource:
                 limit=count,
                 order_by="n_votes",
                 avoid_voted_on=_get_skip_voted_on(auth, device_id),
+                automatically_processable=False,
             )
         )
 
