@@ -1,5 +1,5 @@
 import operator
-from typing import Dict, List, Optional, Set
+from typing import Dict, List, Optional, Set, Tuple
 
 import numpy as np
 
@@ -11,6 +11,7 @@ from robotoff.prediction.types import Prediction, PredictionType
 from robotoff.slack import NotifierFactory
 from robotoff.utils import get_logger, http_session
 from robotoff.utils.cache import CachedStore
+from robotoff.utils.types import JSONType
 
 logger = get_logger(__name__)
 
@@ -23,6 +24,59 @@ LOGO_TYPE_MAPPING: Dict[str, PredictionType] = {
 UNKNOWN_LABEL: LogoLabelType = ("UNKNOWN", None)
 
 
+BoundingBoxType = Tuple[float, float, float, float]
+
+
+def compute_iou(box_1: BoundingBoxType, box_2: BoundingBoxType) -> float:
+    """Compute the IoU (intersection over union) for two bounding boxes.
+
+    The boxes are expected to have the following format:
+    (y_min, x_min, y_max, x_max).
+    """
+    y_min_1, x_min_1, y_max_1, x_max_1 = box_1
+    y_min_2, x_min_2, y_max_2, x_max_2 = box_2
+    x_max = min(x_max_1, x_max_2)
+    x_min = max(x_min_1, x_min_2)
+    y_max = min(y_max_1, y_max_2)
+    y_min = max(y_min_1, y_min_2)
+    width_inter = max(0, x_max - x_min)
+    height_inter = max(0, y_max - y_min)
+    area_inter = width_inter * height_inter
+    box_1_area = (x_max_1 - x_min_1) * (y_max_1 - y_min_1)
+    box_2_area = (x_max_2 - x_min_2) * (y_max_2 - y_min_2)
+    union_area = box_1_area + box_2_area - area_inter
+    return area_inter / union_area
+
+
+def filter_logos(
+    logos: List[JSONType], score_threshold: float, iou_threshold: float = 0.95
+) -> List[Tuple[int, JSONType]]:
+    """Select logos that don't intersect with each other
+    (IoU < `iou_threshold`) and that have a confidence score above
+    `score_threshold`.
+
+    Return a list of (original_idx, logo) tuples.
+    """
+    filtered = []
+    skip_indexes = set()
+    for i in range(len(logos)):
+        logo = logos[i]
+        if i not in skip_indexes:
+            for j in range(i + 1, len(logos)):
+                if (
+                    compute_iou(logo["bounding_box"], logos[j]["bounding_box"])
+                    >= iou_threshold
+                ):
+                    # logos are sorted by descending confidence score, so we ignore
+                    # j logo (logo with lower confidence score)
+                    skip_indexes.add(j)
+
+        if logo["score"] >= score_threshold:
+            filtered.append((i, logo))
+
+    return filtered
+
+
 def get_logo_confidence_thresholds() -> Dict[LogoLabelType, float]:
     thresholds = {}
 
@@ -264,13 +318,16 @@ def generate_insights_from_annotated_logos(
     predictions = []
     for logo in logos:
         prediction = generate_prediction(
-            logo.annotation_type,
-            logo.taxonomy_value,
-            confidence=1.0,
-            logo_id=logo.id,
-            username=logo.username,
-            is_annotation=True,  # it's worth restating it
+            logo_type=logo.annotation_type,
+            logo_value=logo.taxonomy_value,
             automatic_processing=True,  # because this is a user annotation, which we trust.
+            data={
+                "confidence": 1.0,
+                "logo_id": logo.id,
+                "bounding_box": logo.bounding_box,
+                "username": logo.username,
+                "is_annotation": True,  # it's worth restating it
+            },
         )
 
         if prediction is None:
@@ -312,7 +369,13 @@ def predict_logo_predictions(
             continue
 
         prediction = generate_prediction(
-            label[0], label[1], confidence=max_prob, logo_id=logo.id
+            logo_type=label[0],
+            logo_value=label[1],
+            data={
+                "confidence": max_prob,
+                "logo_id": logo.id,
+                "bounding_box": logo.bounding_box,
+            },
         )
 
         if prediction is not None:
@@ -327,10 +390,20 @@ def predict_logo_predictions(
 def generate_prediction(
     logo_type: str,
     logo_value: Optional[str],
+    data: Dict,
     automatic_processing: Optional[bool] = False,
-    **kwargs,
 ) -> Optional[Prediction]:
-    if logo_type not in LOGO_TYPE_MAPPING:
+    """Generate a Prediction from a logo.
+
+    The Prediction may either be created after the annotation of the logo by
+    a human (in which case `automatic_processing` is True), or by infering the
+    logo value from nearest neighbor labels (in which case
+    `automatic_processing` is False).
+
+    Currently, only brand and label logo types are supported: None is returned
+    if the logo type is different, or if the logo_value is None.
+    """
+    if logo_type not in LOGO_TYPE_MAPPING or logo_value is None:
         return None
 
     prediction_type = LOGO_TYPE_MAPPING[logo_type]
@@ -340,13 +413,9 @@ def generate_prediction(
 
     if prediction_type == PredictionType.brand:
         value_tag = value = logo_value
-        if value is None:
-            return None
 
     elif prediction_type == PredictionType.label:
         value_tag = logo_value
-        if value_tag is None:
-            return None
 
     return Prediction(
         type=prediction_type,
@@ -354,5 +423,5 @@ def generate_prediction(
         value=value,
         automatic_processing=automatic_processing,
         predictor="universal-logo-detector",
-        data=kwargs,
+        data=data,
     )
