@@ -77,6 +77,7 @@ DATA_DIR = PROJECT_DIR / "data"
 DATASET_DIR = PROJECT_DIR / "datasets"
 DATASET_DIR.mkdir(exist_ok=True)
 I18N_DIR = PROJECT_DIR / "i18n"
+LABEL_LOGOS_PATH = DATA_DIR / "label_logos.json"
 JSONL_DATASET_PATH = DATASET_DIR / "products.jsonl.gz"
 JSONL_DATASET_ETAG_PATH = DATASET_DIR / "products-etag.txt"
 JSONL_MIN_DATASET_PATH = DATASET_DIR / "products-min.jsonl.gz"
@@ -117,6 +118,15 @@ EVENTS_API_URL = os.environ.get(
     "EVENTS_API_URL", "https://events." + BaseURLProvider().domain
 )
 
+CATEGORY_MATCHER_DIR = DATA_DIR / "category_matcher"
+CATEGORY_MATCHER_MATCH_MAPS = {
+    "category": CATEGORY_MATCHER_DIR / "category_match_maps.json.gz",
+    "ingredient": CATEGORY_MATCHER_DIR / "ingredient_match_maps.json.gz",
+}
+CATEGORY_MATCHER_INTERSECT = (
+    CATEGORY_MATCHER_DIR / "category_ingredient_intersect.json.gz"
+)
+
 # Taxonomies are huge JSON files that describe many concepts in OFF, in many languages, with synonyms. Those are the full version of taxos.
 
 TAXONOMY_DIR = DATA_DIR / "taxonomies"
@@ -161,11 +171,9 @@ ELASTICSEARCH_TYPE = "document"
 
 
 class ElasticsearchIndex:
-    CATEGORY = "category"
     PRODUCT = "product"
 
     SUPPORTED_INDICES = {
-        CATEGORY: (PROJECT_DIR / "robotoff/elasticsearch/index/category_index.json"),
         PRODUCT: (PROJECT_DIR / "robotoff/elasticsearch/index/product_index.json"),
     }
 
